'use strict';

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Function invoked with arguments by NanoEventEmitter.prototype.emit when registered for given eventNames
 *
 * @callback ObserverCallback
 * @param  {*} [...args] zero ore more parameters passed by NanoEventEmitter.prototype.emit
 * @private
 */

/**
 * An EventEmitter
 *
 *
 * @example
 * ```js
 * import NanoEventEmitter from 'nano-event-emitter';
 * const eventEmitter = new NanoEventEmitter();
 * ```
 */

var NanoEventEmitter = function () {
	function NanoEventEmitter() {
		(0, _classCallCheck3.default)(this, NanoEventEmitter);
		this.observers = {};
	}
	/**
  * Attached observers.
  *
  * @private
  */


	(0, _createClass3.default)(NanoEventEmitter, [{
		key: 'on',


		/**
   * Attach an observer for one or more eventNames.
   *
   * @param  {!(string|string[])} eventNames array or string of space seperated event names
   * @param  {ObserverCallback} observer  observer to invoke for eventNames
   * @return {this}
   *
   * @example
   * ```js
   * import NanoEventEmitter from 'nano-event-emitter';
   * const eventEmitter = NanoEventEmitter.create();
   *
   * eventEmitter.on('event', () => { console.log('event'); });
   * eventEmitter.on('more-event', () => { console.log('more-event'); });
   * eventEmitter.on(['even', 'more', 'events'], () => { console.log('even more events'); });
   *
   * eventEmitter.emit(['event', 'more-event']);
   * // console: event
   * // console: more-event
   *
   * eventEmitter.emit('even more events');
   * // console: even more events
   * // console: even more events
   * // console: even more events
   * ```
   */
		value: function on(eventNames, observer) {
			var _this = this;

			var events = Array.isArray(eventNames) ? eventNames : eventNames.split(' ');

			events.filter(Boolean).forEach(function (event) {
				_this.observers[event] = _this.observers[event] || [];
				_this.observers[event].push(observer);
			});

			return this;
		}

		/**
   * Detach an observer for one or more eventNames.
   *
   * @param  {!(string|string[])} eventNames array or string of space seperated event names
   * @param  {?ObserverCallback} [observer] reference to observer function to detach. All observer functions for all eventNames are detached if omitted
   * @return {this}
   *
   * @example
   * ```js
   * import NanoEventEmitter from 'nano-event-emitter';
   * const eventEmitter = NanoEventEmitter.create();
   *
   * const yetAnotherObserver = () => console.log('yet-another-observer');
   *
   * eventEmitter.on('yet-another-event', () => { console.log('yet-another-event'); });
   * eventEmitter.on('yet-another-event', () => { console.log('yet-another-event'); });
   * eventEmitter.on('yet-another-event', yetAnotherObserver);
   *
   * eventEmitter.emit(['yet-another-event']);
   * // console: yet-another-event
   * // console: yet-another-event
   * // console: yet-another-observer
   *
   * eventEmitter.off('yet-another-event', yetAnotherObserver);
   * eventEmitter.emit('yet-another-event');
   * // console: yet-another-event
   * // console: yet-another-event
   *
   * eventEmitter.off('yet-another-event');
   * eventEmitter.emit('yet-another-event');
   * ```
   */

	}, {
		key: 'off',
		value: function off(eventNames, observer) {
			var _this2 = this;

			var events = Array.isArray(eventNames) ? eventNames : eventNames.split(' ');

			if (typeof observer === 'undefined') {
				events.filter(Boolean).forEach(function (event) {
					delete _this2.observers[event];
				});
				return this;
			}

			events.filter(Boolean).forEach(function (event) {
				var observers = _this2.observers[event] || [];
				var index = observers.indexOf(observer);
				if (index > -1) {
					observers.splice(index, 1);
				}
			});

			return this;
		}

		/**
   * Emit one or more events, invoking registered observers with optional args
   *
   * @param  {!(string|string[])} eventNames array or string of space seperated event names
   * @param  {*} [...args] zero ore more parameters to pass to invoked observers
   * @return {this}
   */

	}, {
		key: 'emit',
		value: function emit(eventNames) {
			var _this3 = this;

			for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
				args[_key - 1] = arguments[_key];
			}

			var events = Array.isArray(eventNames) ? eventNames : eventNames.split(' ');

			events.filter(Boolean).forEach(function (event) {
				var observers = _this3.observers[event] || [];
				observers.forEach(function (observer) {
					return observer.apply(undefined, (0, _toConsumableArray3.default)(args || []));
				});
			});

			return this;
		}
	}], [{
		key: 'create',


		/**
   * Create a NanoEventEmitter instance.
   *
   * @return NanoEventEmitter
   *
   * @example
   * ```js
   * import NanoEventEmitter from 'nano-event-emitter';
   * const eventEmitter = NanoEventEmitter.create();
   * ```
   */
		value: function create() {
			for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
				args[_key2] = arguments[_key2];
			}

			return new (Function.prototype.bind.apply(NanoEventEmitter, [null].concat(args)))();
		}
	}]);
	return NanoEventEmitter;
}();

module.exports = NanoEventEmitter;
//# sourceMappingURL=index.js.map